unit AMIPWrapper;

{------------------------------------------------------------------------------}
{                *** Delphi wrapper for the AMIP public API ***                }
{                                                                              }
{ version: 1.3                                                                 }
{ Based on the original ac.h file by Serge Baranov.                            }
{                                                                              }
{ by: Chris Dilts                                                              }
{ last update: July 2, 2008                                                    }
{                                                                              }
{ usage:                                                                       }
{  - Set the DLLPath property to the location of the ac.dll file.              }
{     example: 'x:\path\to\ac.dll'                                             }
{     This is optional and if left blank, or the dll cannot be found at the    }
{     location, the application folder will be searched.                       }                                             
{  - Call the LoadDLL function.                                                }
{     This will attempt to load ac.dll, if found, and return a TDLLLoadError.  }
{  - Do your thing.                                                            }
{     See the AMIP SDK for the usage of the ac_* functions and procedures.     }
{  - No need to unload the DLL manually. It will be automatically unloaded     }
{     on Destroy.                                                              } 
{                                                                              }
{------------------------------------------------------------------------------}

interface

uses
  Windows, Classes;

const
  DLLNAME = 'ac.dll';
  AC_BUFFER_SIZE = 2048;
  { flags for event listener }
  AC_EVT_PLAY = $0001;
  AC_EVT_PAUSE = $0002;
  AC_EVT_STOP = $0004;
  AC_EVT_START = $0008;
  AC_EVT_EXIT = $0010;
  AC_EVT_TIMER = $0020;
  AC_EVT_MSG = $0040;
  AC_EVT_CHANGE = $0080;
  AC_EVT_PLCHANGE = $0100;
  AC_EVT_PLREADY = $0200;
  AC_EVT_ALL = AC_EVT_PLAY + AC_EVT_PAUSE + AC_EVT_STOP + AC_EVT_START + AC_EVT_EXIT + AC_EVT_CHANGE + AC_EVT_PLCHANGE + AC_EVT_PLREADY;

type
  TDLLLoadError = (
    ERR_DLL_NOERROR,
    ERR_DLL_ALREADYLOADED,
    ERR_DLL_NOTFOUND,
    ERR_DLL_LOADFAILURE );

  Tac_StartMode = (
    AC_START_ALL,
    AC_START_CLIENT,
    AC_START_SERVER,
    AC_START_NONE );

  Tac_ErrorCode = (
    AC_ERR_NOERROR,
    AC_ERR_CLIENTISNULL,
    AC_ERR_EXCEPTION,
    AC_ERR_CONNECTIONFAILED,
    AC_ERR_SERVERNOTRUNNING );

  Tac_msg_callback = procedure(const msg: pchar); stdcall;
  Tac_evt_callback = procedure(evt: integer); stdcall;

  TAMIPWrapper = class(TComponent)
  private
    DLLLoaded: boolean;
    DLLHandle: THandle;
    fDLLPath: string;
    procedure SetDLLPath(const aPath: string);
    function FindDLL: boolean;
  public
    { client }
    ac_init_client: function(const host: pchar; port, timeout, dsec, dcount: integer): integer; stdcall;
    ac_uninit_client: procedure; stdcall;
    ac_setDestPort: procedure(port: integer); stdcall;
    ac_setDestHost: procedure(const host: pchar); stdcall;
    ac_setTimeout: procedure(ms: integer); stdcall;
    ac_setSrcPort: procedure(port: integer); stdcall;
    ac_setSrcHost: procedure(const host: pchar); stdcall;
    ac_getSrcPort: function: integer; stdcall;
    ac_getDestPort: function: integer; stdcall;
    ac_getSrcHost: procedure(out_host: pchar); stdcall;
    ac_getDestHost: procedure(out_host: pchar); stdcall;
    ac_rehash: procedure; stdcall;
    ac_is_client_initialized: function(): integer; stdcall;
    { server }
    ac_init_server: function(const host: pchar; port: integer): integer; stdcall;
    ac_stop_server: procedure; stdcall;
    ac_pingServer: function(const host: pchar; port, timeout: integer): boolean; stdcall;
    ac_is_server_started: function(): integer; stdcall;
    { listener and callbacks }
    ac_add_event_listener: function(const host: pchar; port, timeout: integer; flags: word; fail_count: word = 1): Tac_ErrorCode; stdcall;
    ac_remove_event_listener: function(const host: pchar; port: integer): Tac_ErrorCode; stdcall;
    ac_register_msg_callback: procedure(_1: Tac_msg_callback); stdcall;
    ac_register_evt_callback: procedure(_1: Tac_evt_callback); stdcall;
    { general }
    ac_init: procedure(mode: Tac_StartMode = AC_START_ALL); stdcall;
    ac_uninit: procedure; stdcall;
    ac_exec: function(const cmd: pchar): Tac_ErrorCode; stdcall;
    ac_eval: function(const cmd: pchar; res: pchar): Tac_ErrorCode; stdcall;
    ac_format: function(const cmd: pchar; res: pchar): Tac_ErrorCode; stdcall;
    { playlist functions }
    ac_get_pl: function: Tac_ErrorCode; stdcall;
    ac_get_plsize: function: integer; stdcall;
    ac_get_title: function(idx: word; buff: pchar): integer; stdcall;
    { version info }
    ac_version_major: function: integer; stdcall;
    ac_version_minor: function: integer; stdcall;
    function LoadDLL: TDLLLoadError;
    function UnloadDLL: boolean;    
  published
    constructor Create(Owner: TComponent); override;
    destructor Destroy; override;
    property DLLPath: string read fDLLPath write SetDLLPath;
  end;

procedure Register;

implementation

uses
  SysUtils;

procedure Register;
begin
  RegisterComponents('blert Tools', [TAMIPWrapper]);
end;

constructor TAMIPWrapper.Create(Owner: TComponent);
begin
  inherited;
  DLLLoaded := false;
end;

destructor TAMIPWrapper.Destroy;
begin
  if DLLLoaded then UnloadDLL;
  inherited;
end;

function TAMIPWrapper.FindDLL: boolean;
var
  pathBuff: string;
begin
  Result := false;
  //determine if DLLPath holds a valid path to ac.dll
  if fDLLPath <> '' then
    if ExtractFileName(fDLLPath) = DLLNAME then
      if FileExists(fDLLPath) then
        begin
          Result := true;
          Exit;
        end;
  //determine if ac.dll resides in the application folder
  if FileExists(DLLNAME) then
    begin
      Result := true;
      SetDLLPath(DLLNAME);
      Exit;
    end;
end;

function TAMIPWrapper.LoadDLL: TDLLLoadError;
begin
  Result := ERR_DLL_NOERROR;
  if DLLLoaded then
    begin
      Result := ERR_DLL_ALREADYLOADED;
      Exit;
    end;
  if FindDLL then
    begin
      DLLHandle := LoadLibrary(PChar(fDLLPath));
      if DLLHandle <> 0 then
        begin
          DLLLoaded := True;
          @ac_init_client := GetProcAddress(DLLHandle,'ac_init_client');
          @ac_init_server := GetProcAddress(DLLHandle,'ac_init_server');
          @ac_init := GetProcAddress(DLLHandle,'ac_init');
          @ac_uninit := GetProcAddress(DLLHandle,'ac_uninit');
          @ac_uninit_client := GetProcAddress(DLLHandle,'ac_uninit_client');
          @ac_stop_server := GetProcAddress(DLLHandle,'ac_stop_server');
          @ac_exec := GetProcAddress(DLLHandle,'ac_exec');
          @ac_eval := GetProcAddress(DLLHandle,'ac_eval');
          @ac_format := GetProcAddress(DLLHandle,'ac_format');
          @ac_register_msg_callback := GetProcAddress(DLLHandle,'ac_register_msg_callback');
          @ac_register_evt_callback := GetProcAddress(DLLHandle,'ac_register_evt_callback');
          @ac_add_event_listener := GetProcAddress(DLLHandle,'ac_add_event_listener');
          @ac_remove_event_listener := GetProcAddress(DLLHandle,'ac_remove_event_listener');
          @ac_pingServer := GetProcAddress(DLLHandle,'ac_pingServer');
          @ac_get_pl := GetProcAddress(DLLHandle,'ac_get_pl');
          @ac_get_plsize := GetProcAddress(DLLHandle,'ac_get_plsize');
          @ac_get_title := GetProcAddress(DLLHandle,'ac_get_title');
          @ac_setDestPort := GetProcAddress(DLLHandle,'ac_setDestPort');
          @ac_setDestHost := GetProcAddress(DLLHandle,'ac_setDestHost');
          @ac_setTimeout := GetProcAddress(DLLHandle,'ac_setTimeout');
          @ac_setSrcPort := GetProcAddress(DLLHandle,'ac_setSrcPort');
          @ac_setSrcHost := GetProcAddress(DLLHandle,'ac_setSrcHost');
          @ac_getSrcPort := GetProcAddress(DLLHandle,'ac_getSrcPort');
          @ac_getDestPort := GetProcAddress(DLLHandle,'ac_getDestPort');
          @ac_getSrcHost := GetProcAddress(DLLHandle,'ac_getSrcHost');
          @ac_getDestHost := GetProcAddress(DLLHandle,'ac_getDestHost');
          @ac_rehash := GetProcAddress(DLLHandle,'ac_rehash');
          @ac_is_server_started := GetProcAddress(DLLHandle,'ac_is_server_started');
          @ac_is_client_initialized := GetProcAddress(DLLHandle,'ac_is_client_initialized');
          @ac_version_major := GetProcAddress(DLLHandle,'ac_version_major');
          @ac_version_minor := GetProcAddress(DLLHandle,'ac_version_minor');
        end
      else
        begin
          DLLLoaded := False;
          Result := ERR_DLL_LOADFAILURE;
        end;
    end
  else
    begin
      DLLLoaded := False;
      Result := ERR_DLL_NOTFOUND;
    end;
end;

function TAMIPWrapper.UnloadDLL: boolean;
begin
  Result := false;
  if FreeLibrary(DLLHandle) then
    begin
      DLLLoaded := False;
      Result := true;
    end;
end;

procedure TAMIPWrapper.SetDLLPath(const aPath: string);
begin
  fDLLPath := aPath;
end;

{------------------------------------------------------------------------------}
{                              *** Change Log ***                              }
{                                                                              }
{ v 1.3                                                                        }
{   - added: ac_getSrcPort, ac_getDestPort, ac_getSrcHost, ac_getDestHost,     }
{     ac_is_server_started, ac_is_client_initialized                           }
{   - added automatic DLL unload on Destroy                                    }
{   - removed seach for ac.dll in the AMIPconfigurator folder                  }
{   - minor changes and some clean-up                                          }
{                                                                              }
{ v 1.2                                                                        }
{   - your guess is as good as mine                                            }
{                                                                              }
{ v 1.1                                                                        }
{   - added playlist functions                                                 }
{                                                                              }
{ v 1.0                                                                        }
{   - first incarnation                                                        }
{                                                                              }
{------------------------------------------------------------------------------}

end.
